; 3DFILLET.LSP                        1/1/95
;    For AutoCAD(tm) Release 12
;
; Redefines the FILLET command to allow filleting
; of non-UCS coplanar lines, and allows radius
; definition during FILLET
;
;
; ***************************************
; ****  Author:  Owen Wengerd        ****
; ****                               ****
; ****  Manu-Soft Computer Services  ****
; ****  P.O. Box 84                  ****
; ****  Fredericksburg, OH  44627    ****
; ****  (216) 695-5903               ****
; ****  Compu-Serve ID:  71324,3252  ****
; ***************************************

(if (not c:fillet) (command "._UNDEFINE" "_FILLET"))

(defun C:FILLET (/ 

;*** Local Variables ***

  ent1
  ent2
  inp
  olderr
  oldvar 
  p1
  p2
  restore
  tmp1
  tmp2
  vertlist


;*** Local Functions ***

  dxf
  equalf
  errexit
  filletx
  findparent
  get_input
  is_valid
  zerof

)

;*** Local Function Definitions ***

(defun errexit (s)
  (if (= 8 (logand 8 (getvar "UNDOCTL"))) 
    (command "UNDO" "E" "UNDO" 1)
  )
  (if (not (member s '("console break" "Function cancelled")))
    (princ (strcat "\nError:  " s))
  )
  (restore)
)

(defun filletx (/ t1)
  (if (/= 1 (setq t1 (logand 3 (nth 1 oldvar))))
    (progn
      (command "_UNDO")
      (if (/= 0 (logand 3 (getvar "UNDOCTL"))) (command "_C"))
      (command (if (= 0 t1) "_N" "_O"))
    )
  )
  (setvar "UCSFOLLOW" (nth 4 oldvar))
  (setvar "UCSICON" (nth 3 oldvar))
  (setvar "BLIPMODE" (nth 2 oldvar))
  (setvar "CMDECHO" (car oldvar))
  (setq *error* olderr)
  (princ)
)

(defun dxf (a e) (cdr (assoc a e)))

(defun get_input (prmpt kwrd / inp)
  (while (not inp)
    (initget (strcat "Radius " kwrd))
    (setq inp
      (entsel
        (strcat
          "\nRadius ("
          (rtos (getvar "FILLETRAD"))
          ")/"
          prmpt
          ": "
        )
      )
    )
    (cond 
      ( (= inp "Radius")
        (if
          (setq inp
            (getreal
              (strcat
                "Enter fillet radius <"
                (rtos (getvar "FILLETRAD"))
                ">: "
              )
            )
          )
          (setvar "FILLETRAD" inp)
        )
        (setq inp nil)
      )
      ( inp
        (if (listp inp) (setq inp (nentselp (cadr inp))))
      )
      ( T
        (setq inp 0)
      )
    )
  )
  (if (not (numberp inp)) inp)
)

(defun is_valid (elist vert / etype)
  (setq etype (dxf 0 elist))
  (not
    (cond 
      ( (= etype "VERTEX")
        (cond 
          ( (and vert (listp vert))
            (cond 
              ( (not (zerop (dxf 42 elist)))
                (princ "\nCannot fillet polyline arc segments!")
              )
              ( (not (member (dxf -1 elist) vert))
                (princ
                  (strcat
                    "\nLines belonging to polylines must be consecutive"
                    " or separated by one segment!"
                  )
                )
              )
            )
          )
          ( vert
            (princ
              (strcat "\nCannot fillet polyline segments"
                      " with non-polyline segments!"
              )
            )
          )
          ( T
            (cond
              ( (>= (dxf 70 elist) 128)
                (princ "\n3D Polyface Mesh entities may not be filleted!")
              )
              ( (>= (dxf 70 elist) 64)
                (princ "\n3D Polygon Mesh entities may not be filleted!")
              )
              ( (>= (dxf 70 elist) 32)
                (princ "\n3D Polyline entities may not be filleted!")
              )
              ( (not (zerop (dxf 42 elist)))
                (princ "\nCannot fillet polyline arc segments!")
              )
            )
          )
        )
      )
      ( (member etype '("LINE" "ARC" "CIRCLE")) 
        (if (and vert (listp vert))
          (princ
            (strcat "\nCannot fillet polyline segments"
                    " with non-polyline segments!"
            )
          )
        )
      )
      ( T
        (princ 
          (strcat "\nFillet requires lines, arcs, circles"
                  ", or straight polyline segments!"
          )
        )
      )
    )
  )
)

(defun findparent (ent / edata)
  (while
    (and
      (setq ent (entnext ent))
      (/= (dxf 0 (setq edata (entget ent))) "SEQEND")
    )
  )
  (if ent (cdr (assoc -2 edata)))
)

(defun equalf (real1 real2)
  (equal real1 real2 0.0000000001)
)

(defun zerof (realnum)
  (equal realnum 0 0.0000000001)
)


; ***********************************************
; ***************  Main Program  ****************
; ***********************************************

  (setq T (not nil))
  (setq oldvar
    (list
      (getvar "CMDECHO")
      (getvar "UNDOCTL")
      (getvar "BLIPMODE")
      (getvar "UCSICON")
      (getvar "UCSFOLLOW")
    )
  )
  (setvar "CMDECHO" 0)
  (setvar "BLIPMODE" 0)
  (setq restore filletx
        olderr  *error*
        *error* errexit
  )
  (while (and (not inp) (setq inp (get_input "<Select first object>" "")))
    (and
      (if (not (is_valid (setq ent1 (entget (car inp))) nil))
        (setq inp nil)
        T
      )
      (if (> (length inp) 2)
        (progn
          (initget "Yes No")
          (if
            (= "Yes"
              (getkword
                (strcat
                  "\nYou selected an entity in a block!"
                  "\n    Do you want to explode the block? (Yes/<No>): "
                )
              )
            )
            (progn
              (while (> (length inp) 2)
                (command "._EXPLODE" (last (last inp)))
                (setq inp (nentselp (cadr inp)))
              )
              (setq ent1 (entget (car inp)))
            )
          )
        )
        T
      )
      (progn
        (redraw (car inp) 3)
        (cond
          ( (= (dxf 0 ent1) "VERTEX")
            (setq p1   (cadr inp)
                  tmp1 (findparent (dxf -1 ent1))
                  tmp2 tmp1
            )
            (while (/= (dxf 0 (entget (setq tmp1 (entnext tmp1)))) "SEQEND")
              (setq vertlist (cons tmp1 vertlist))
            )
            (setq vertlist
              (list
                (cadr (member (dxf -1 ent1) vertlist))
                (caddr (member (dxf -1 ent1) vertlist))
                (cadr (member (dxf -1 ent1) (reverse vertlist)))
                (caddr (member (dxf -1 ent1) (reverse vertlist)))
              )
            )
            (setq inp nil)
            (while (not inp)
              (setq inp
                (get_input
                  "Undo/Select second segment(<CR>=entire polyline)"
                  "Undo"
                )
              )
              (cond
                ( (= inp "Undo")
                  (redraw (dxf -1 ent1) 4)
                )
                ( (null inp)
                  (redraw (dxf -1 ent1) 4)
                  (setvar "UCSICON" 0)
                  (command 
                    "._UCS"
                    "_E"
                    tmp2
                    "._FILLET"
                    "_P"
                  )
                  (setvar "CMDECHO" 1)
                  (command tmp2)
                  (setvar "CMDECHO" 0)
                  (command
                    "._UCS"
                    "_P"
                  )
                  (setvar "UCSICON" (nth 3 oldvar))                
                  (setq inp T)
                )  
                ( T 
                  (if (is_valid (setq ent2 (entget (car inp))) vertlist)
                    (progn
                      (redraw (dxf -1 ent1) 4)
                      (setvar "UCSICON" 0)
                      (setq p1 (trans p1 1 0)
                            p2 (trans (cadr inp) 1 0)
                      )
                      (command 
                        "._UCS"
                        "_E"
                        tmp2
                        "._FILLET"
                        (trans p1 0 1)
                        (trans p2 0 1)
                        "._UCS"
                        "_P"
                      )
                      (setvar "UCSICON" (nth 3 oldvar))                
                    )
                    (setq inp nil)
                  )
                )
              )
            )
            (if (= inp "Undo") (setq inp nil))
          )
          ( T
            (setq ent1 (list (dxf 0 ent1) (car inp) (trans (cadr inp) 1 0))
                  inp  nil
            )
            (while (not inp)
              (setq inp (get_input "Undo/Select second object" "Undo"))
              (cond 
                ( (= inp "Undo")
                  (redraw (cadr ent1) 4)
                )
                ( (null inp)
                  (setq inp 1)
                  (redraw (cadr ent1) 4)
                )
                ( (is_valid (setq ent2 (entget (car inp))) T)
                  (setq ent2 (dxf 0 ent2))
                  (cond
                    ( (= (car ent1) ent2 "LINE")
                      (if
                        (inters
                          (dxf 10 (entget (cadr ent1)))
                          (dxf 11 (entget (cadr ent1)))
                          (dxf 10 (entget (car inp)))
                          (dxf 11 (entget (car inp)))
                          nil
                        )
                        (progn
                          (setq ent2
                            (cons
                              ent2
                              (list (car inp) (trans (cadr inp) 1 0))
                            )
                          )
                          (setq tmp1 (entget (cadr ent1))
                                tmp2 (entget (cadr ent2))
                          )
                          (if
                            (and
                              (equalf
                                (caddr (trans (dxf 10 tmp1) 0 1))
                                (caddr (trans (dxf 11 tmp1) 0 1))
                              )
                              (equalf
                                (caddr (trans (dxf 11 tmp1) 0 1))
                                (caddr (trans (dxf 10 tmp2) 0 1))
                              )
                              (equalf
                                (caddr (trans (dxf 10 tmp2) 0 1))
                                (caddr (trans (dxf 11 tmp2) 0 1))
                              )
                            )
                            (command
                              "._FILLET"
                              (trans (caddr ent1) 0 1)
                              (trans (caddr ent2) 0 1)
                            )
                            (progn
                              (setvar "UCSICON" 0)
                              (command
                                "._UCS"
                                "_3"
                                (trans (dxf 10 tmp1) 0 1)
                                (trans (dxf 11 tmp1) 0 1)
                                (trans
                                  (if
                                    (or
                                      (equalf
                                        (dxf 10 tmp2)
                                        (dxf 10 tmp1)
                                      )
                                      (equalf
                                        (dxf 10 tmp2)
                                        (dxf 11 tmp1)
                                      )
                                    )
                                    (dxf 11 tmp2)
                                    (dxf 10 tmp2)
                                  )
                                  0
                                  1
                                )
                                "._FILLET"
                                (trans (caddr ent1) 0 1)
                                (trans (caddr ent2) 0 1)
                                "._UCS"
                                "_P"
                              )
                              (setvar "UCSICON" (nth 3 oldvar))
                            )
                          )
                        )
                        (prompt "\nEntities are not co-planar!")
                      )
                    )
                    ( (or (setq tmp1 (= (car ent1) "LINE")) (= ent2 "LINE"))
                      (setq ent2
                        (cons ent2 (list (car inp) (trans (cadr inp) 1 0)))
                      )
                      (setvar "UCSICON" 0)
                      (command
                        "._UCS"
                        "_E"
                        (if tmp1 (cadr ent2) (cadr ent1))
                      )
                      (if
                        (and
                          (zerof
                            (last
                              (trans
                                (dxf 10
                                  (entget (cadr (if tmp1 ent1 ent2)))
                                )
                                0
                                1
                              )
                            )
                          )
                          (zerof
                            (last
                              (trans
                                (dxf 11
                                  (entget (cadr (if tmp1 ent1 ent2))) 
                                )
                                0
                                1
                              )
                            )
                          )
                        )
                        (command
                          "._FILLET"
                          (trans (caddr ent1) 0 1)
                          (trans (caddr ent2) 0 1)
                        )
                        (prompt "\nEntities are not co-planar!")
                      )
                      (command
                        "._UCS"
                        "_P"
                      )
                      (setvar "UCSICON" (nth 3 oldvar))
                    )
                    ( (equal
                        (dxf 210 (entget (cadr ent1)))
                        (dxf 210 (entget (car inp)))
                      )
                      (setvar "UCSICON" 0)
                      (command
                        "._UCS"
                        "_E"
                        (cadr ent1)
                        "._FILLET"
                        (trans (caddr ent1) 0 1)
                        (trans (cadr inp) 0 1)
                        "._UCS"
                        "_P"
                      )
                      (setvar "UCSICON" (nth 3 oldvar))
                    )
                    ( T (prompt "\nEntities are not co-planar!"))
                  )
                  (if inp (redraw (cadr ent1) 4))
                )
                ( T (setq inp nil))
              )
            )
            (setq inp (if (= inp "Undo") nil 1))
          )
        )
      )
    )
  )
  (restore)
)

